
require("Actors/AI/PID")

function Create(self)
	---------------- AI variables start ----------------
	self.StuckTimer = Timer()
	self.HatchTimer = Timer()
	
	self.AvoidTimer = Timer()
	self.AvoidTimer:SetSimTimeLimitMS(500)
	
	self.PlayerInterferedTimer = Timer()
	self.PlayerInterferedTimer:SetSimTimeLimitMS(500)
	
	self.Ctrl = self:GetController()
	self.LastAIMode = Actor.AIMODE_NONE
	
	-- The drop ship tries to hover this many pixels above the ground
	if self.AIMode == Actor.AIMODE_BRAINHUNT then
		self.hoverAlt = self.Radius * 1.7
	elseif self.AIMode == Actor.AIMODE_GOTO then
		self.hoverAlt = self.Radius * 7
	else
		self.hoverAlt = self.Radius * 2	
	end
	
	-- The controllers
	self.XposPID = RegulatorPID:New{p=0.05, i=0.01, d=2.5, filter_leak=0.8, integral_max=50}
	self.YposPID = RegulatorPID:New{p=0.1, d=2.5, filter_leak=0.6}
	
	---------------- AI variables end ----------------
	
	
	function GSOffset(Origin, ang, dist)
		if Origin then
			return Vector(Origin.X, Origin.Y) + Vector(dist,0):RadRotate(ang)	-- this way it does not matter if Origin is a const vector
		end
		return Vector(dist,0):RadRotate(ang)
	end
	
	self.GunTimer = Timer()
	self.FlippTimer = Timer()
	self.DoorTimer = Timer()
	self.ScuttleTimer = Timer()
	self.door = 0
	self.maxFrame = 6	-- #of frames - 1
	self.flipp = 0
	self.aim = self.RotAngle
	self.AimDots = {}
	self.SharpLength = Vector(FrameMan.PlayerScreenWidth/2.7, FrameMan.PlayerScreenHeight/3)
	self.AnalogAim = self.Ctrl:IsMouseControlled()
end

function Destroy(self)
	if MovableMan:IsParticle(self.Barrel) then
		self.Barrel.ToDelete = true
	end

	for i = 1, 3 do
		if MovableMan:IsParticle(self.AimDots[i]) then
			self.AimDots[i].ToDelete = true
		end
	end

	if MovableMan:IsParticle(self.PayloadIcon) then
		self.PayloadIcon.ToDelete = true
	end

	if MovableMan:IsParticle(self.MGIcon) then
		self.MGIcon.ToDelete = true
	end
end

function Update(self)
	if self.Health < 1 or (self.ToDelete and self.Pos.Y > -50) then
		self:GibThis()
		
		local HullGib = CreateMOSRotating("Transport Gib", "Air Support.rte")
		if HullGib then
			HullGib.Pos = self.Pos
			HullGib.Vel = self.Vel
			HullGib.RotAngle = self.RotAngle
			HullGib.HFlipped = self.HFlipped
			HullGib.AngularVel = self.AngularVel * 0.7
			MovableMan:AddParticle(HullGib)
		end
	elseif not self.EngineLeft then
		-- Find the engines
		local MoObj
		for i = 1, MovableMan:GetMOIDCount()-1 do
			if i ~= self.ID and MovableMan:GetRootMOID(i) == self.ID then	-- an obj without a parent has ID == RootID
				MoObj = MovableMan:GetMOFromID(i)
				if MoObj.PresetName == "Transport Engine Left" then
					self.EngineLeft = ToAEmitter(MoObj)
				elseif MoObj.PresetName == "Transport Engine Right" then
					self.EngineRight = ToAEmitter(MoObj)
				end
			end
		end
	end

	-- Reduce the risk of auto scuttling
	if self.RotAngle > 6.2832 then
		self.RotAngle = self.RotAngle - 6.2832
	elseif self.RotAngle < -1.5708 then
		self.RotAngle = self.RotAngle + 6.2832
	end

	local gunAim = 0
	local GunOffset = self.Pos + self:RotateOffset(Vector(52, 25))
	
	-- Validate barrel
	if not MovableMan:IsParticle(self.Barrel) or self.Barrel.PresetName ~= "Transport MG Barrel" then
		self.Barrel = CreateAEmitter("Transport MG Barrel", "Air Support.rte")
		MovableMan:AddParticle(self.Barrel)
		self.Barrel.Pos = GunOffset
		self.Barrel.RotAngle = gunAim
		self.Barrel:EnableEmission(false)
	end
	
	-- Door animations
	if self.door == 0 then
		if self.Frame > 0 and self.Frame < self.maxFrame and self.DoorTimer:IsPastSimMS(80) then
			self.DoorTimer:Reset()
			
			if self.closeDoors then
				self.Frame = self.Frame - 1
			else
				self.Frame = self.Frame + 1
			end
		end
	elseif self.DoorTimer:IsPastSimMS(250) then
		self.door = 0
	end
	
	if self:IsPlayerControlled() then
		self.player = true
		
		-- Don't open the doors with the fire button
		self.Ctrl:SetState(Controller.PRESS_FACEBUTTON, false)
		
		-- Tripple tapp down key to open the doors
		if self.Ctrl:IsState(Controller.PRESS_DOWN) then
			self.DoorTimer:Reset()
			self.door = self.door + 1
			if self.door > 2 then
				self.Ctrl:SetState(Controller.PRESS_FACEBUTTON, true)	-- Open the doors
				self.door = 0
				
				if self.Frame == 0 then
					self.Frame = 1
					self.closeDoors = false
					self.DoorTimer:Reset()
				else
					self.Frame = self.maxFrame - 1
					self.closeDoors = true
					self.DoorTimer:Reset()
				end
			end
		end
		
		-- Double or tripple tapp movement keys to flipp
		if self.flipp ~= 0 and self.FlippTimer:IsPastSimMS(200) then
			self.flipp = 0
		end
		
		if self.HFlipped then
			if self.Ctrl:IsState(Controller.PRESS_RIGHT) then
				self.FlippTimer:Reset()
				self.flipp = self.flipp + 1
				if self.Ctrl:IsMouseControlled() then
					if self.flipp > 1 then
						self.flipp = nil
					end
				elseif self.flipp > 2 then
					self.flipp = nil
				end
			end
		elseif self.Ctrl:IsState(Controller.PRESS_LEFT) then
			self.FlippTimer:Reset()
			self.flipp = self.flipp - 1
			if self.Ctrl:IsMouseControlled() then
				if self.flipp < -1 then
					self.flipp = nil
				end
			elseif self.flipp < -2 then
				self.flipp = nil
			end
		end
		
		if not self.flipp then
			self.HFlipped = not self.HFlipped
			self.aim = 3.1415 - self.aim
			self.flipp = 0
			
			if self.EngineLeft.ID < 255 and self.EngineLeft:IsAttached() then
				self.EngineLeft.HFlipped = not self.EngineLeft.HFlipped
			end
			
			if self.EngineRight.ID < 255 and self.EngineRight:IsAttached() then
				self.EngineRight.HFlipped = not self.EngineRight.HFlipped
			end
		end
		
		-- Aim
		if self.AnalogAim then
			self.aim = self.Ctrl.AnalogAim.AbsRadAngle
			if self.HFlipped then
				if self.aim > 1.57 + self.RotAngle then
					self.aim = math.max(self.aim, self.RotAngle + 2.89)
				else
					self.aim = math.min(self.aim, self.RotAngle - 0.94)
				end
			else
				if self.aim > 1.57 + self.RotAngle then
					self.aim = math.max(self.aim, self.RotAngle + 4.1)
				else
					self.aim = math.min(self.aim, self.RotAngle + 0.25)
				end
			end
			
			gunAim = self.aim
		else
			if self.Ctrl:IsState(Controller.BODY_JUMP) then
				if self.HFlipped then
					self.aim = self.aim - 0.04
				else
					self.aim = self.aim + 0.04
				end
			elseif self.Ctrl:IsState(Controller.BODY_CROUCH) then
				if self.HFlipped then
					self.aim = self.aim + 0.04
				else
					self.aim = self.aim - 0.04
				end
			elseif self.Ctrl.AnalogAim.Magnitude > 0 then
				self.AnalogAim = true
			end
			
			if self.aim > 6.2832 then
				self.aim = self.aim - 6.2832
			elseif self.aim < -1.5708 then
				self.aim = self.aim + 6.2832
			end
			
			if self.HFlipped then
				if self.aim > 1.57 then
					self.aim = math.max(self.aim, 2.89)
				else
					self.aim = math.min(self.aim, -0.34)
				end
			else
				if self.aim > 1.57 then
					self.aim = math.max(self.aim, 3.5)
				else
					self.aim = math.min(self.aim, 0.25)
				end
			end
			
			gunAim = self.RotAngle + self.aim
		end
		
		-- Update the aim
		for i = 1, 3 do
			if not MovableMan:IsParticle(self.AimDots[i]) or self.AimDots[i].PresetName ~= "Aim" then
				self.AimDots[i] = CreateMOSParticle("Aim", "Air Support.rte")
				MovableMan:AddParticle(self.AimDots[i])
			end
			
			self.AimDots[i].Age = 1
			self.AimDots[i].Vel = self.Vel
			self.AimDots[i].Pos = GSOffset(GunOffset, gunAim, 190 + i*10)
			
			if SceneMan.SceneWrapsX and self.AimDots[i].Pos.X > SceneMan.SceneWidth then
				self.AimDots[i].Pos.X = self.AimDots[i].Pos.X - SceneMan.SceneWidth
			end
		end
		
		if self.Ctrl:IsState(Controller.WEAPON_FIRE) then
			if self.GunTimer:IsPastSimMS(80) then
				self.GunTimer:Reset()

				local Casing = CreateMOSParticle("Gunship MG Casing", "Air Support.rte")
				if Casing then
					Casing.Pos = GunOffset + self.Barrel:RotateOffset(Vector(5, 7))
					Casing.RotAngle = self.Barrel.RotAngle + RangeRand(-0.5,0.5)
					Casing.AngularVel = RangeRand(-2,12)
					
					if self.HFlipped then
						Casing.Vel = GSOffset(self.Vel * RangeRand(0.8,0.9), gunAim + RangeRand(1.1,1.5), 5)
					else
						Casing.Vel = GSOffset(self.Vel * RangeRand(0.8,0.9), gunAim - RangeRand(1.1,1.5), 5)
					end
					
					MovableMan:AddParticle(Casing)
				end
				
				self.Barrel:EnableEmission(true)
				self.Barrel:TriggerBurst()
			end
		else
			self.Barrel:EnableEmission(false)
		end
		
		self.ViewPoint = self.Pos + Vector(math.cos(gunAim) * self.SharpLength.X, -math.sin(gunAim) * self.SharpLength.Y)
	elseif self.player then
		self.player = false	-- only execute this code once after the player has switched actor
		
		for i = 1, 3 do
			if MovableMan:IsParticle(self.AimDots[i]) then
				self.AimDots[i].ToDelete = true
			end
		end
		
		if MovableMan:IsParticle(self.PayloadIcon) then
			self.PayloadIcon.ToDelete = true
		end
		
		if MovableMan:IsParticle(self.MGIcon) then
			self.MGIcon.ToDelete = true
		end
	else	-- AI mode
		-- Hover assist
		self.RotAngle = self.RotAngle * 0.993
		self.Vel.X = self.Vel.X * 0.98
		if math.abs(self.Vel.Y) > 10 then
			self.Vel.Y = self.Vel.Y * 0.95
		end
		
		-- Reduce the risk of auto scuttling
		if self.Vel.Magnitude > 5 then
			self.ScuttleTimer:Reset()
		elseif self.ScuttleTimer:IsPastSimMS(7000) then
			self.Vel.Y = self.Vel.Y - 0.4
		end
		
		-- The AI just opened the doors, play the animation
		if self.Ctrl:IsState(Controller.PRESS_FACEBUTTON) then
			if self.Frame == 0 then
				self.door = 0
				self.Frame = 1
				self.closeDoors = false
				self.DoorTimer:Reset()
			elseif self.Frame == self.maxFrame then
				self.door = 0
				self.Frame = self.maxFrame - 1
				self.closeDoors = true
				self.DoorTimer:Reset()
			else
				self.Ctrl:SetState(Controller.PRESS_FACEBUTTON, false)
			end
		end
		
		-- Move aim towards zero
		if self.HFlipped then
			if self.aim > 3.16 or self.aim < 0 then
				self.aim = self.aim - 0.03
			elseif self.aim < 3.12 then
				self.aim = self.aim + 0.03
			end
		else
			if self.aim < -0.02 or self.aim > 3.14 then
				self.aim = self.aim + 0.03
			elseif self.aim > 0.02 then
				self.aim = self.aim - 0.03
			end
		end
		
		if self.aim > 6.2832 then
			self.aim = self.aim - 6.2832
		elseif self.aim < -1.5708 then
			self.aim = self.aim + 6.2832
		end
		
		gunAim = self.RotAngle + self.aim
	end
	
	if self.EngineLeft.ID < 255 and self.EngineLeft:IsAttached() and
		self.EngineRight.ID < 255 and self.EngineRight:IsAttached()
	then	-- Make sure the engines are attached
		local Acc = Vector()
		local power = {12000, 1200, 5000, 35000, 9000, 20000}
		
		-- Rotate
		if self.Ctrl:IsState(Controller.MOVE_LEFT) and self.AngularVel < 2 then
			if self.HFlipped then
				Acc:SetMagnitude(power[1]) -- Facing left; accelerate
			else
				if self.RotAngle < 0 then
					Acc:SetMagnitude(power[2])	-- Facing right; do not break hard until level
				else
					Acc:SetMagnitude(power[3])	-- Facing right; break
				end
			end
			
			Acc:RadRotate(self.RotAngle/2 + 3.141592)
			self.AngularVel = self.AngularVel + 0.124
		elseif self.Ctrl:IsState(Controller.MOVE_RIGHT) and self.AngularVel > -2 then
			if self.HFlipped then
				if self.RotAngle > 0 then
					Acc:SetMagnitude(power[2])	-- Facing left; do not break hard until level
				else
					Acc:SetMagnitude(power[3])	-- Facing left; break
				end
			else
				Acc:SetMagnitude(power[1]) -- Facing right; accelerate
			end
			
			self.AngularVel = self.AngularVel - 0.124
			Acc:RadRotate(self.RotAngle/2)
		else
			self.AngularVel = self.AngularVel * 0.96
		end
		
		-- Lift
		local Lft = Vector(0, 0)
		if self.Ctrl:IsState(Controller.MOVE_UP) then
			Lft:SetMagnitude(power[4])
		elseif self.Ctrl:IsState(Controller.MOVE_DOWN) then
			Lft:SetMagnitude(power[5])
		else
			Lft:SetMagnitude(power[6])
		end
		
		Lft:RadRotate(self.RotAngle)
		self:AddAbsForce(Lft.Perpendicular + Acc + Vector(0, -1500), self.Pos)
	elseif self.Health > 0 then
		self.Health = self.Health - 1
	end

	-- Update barrel
	self.Barrel.Pos = GunOffset
	self.Barrel.RotAngle = gunAim
	self.Barrel.HFlipped = self.HFlipped
	
	if self.HFlipped then
		self.Barrel.RotAngle = self.Barrel.RotAngle + 3.141592
	end
end

function UpdateAI(self)
	if self.AIMode ~= self.LastAIMode then	-- We have new orders
		self.LastAIMode = self.AIMode
		
		if self.AIMode == Actor.AIMODE_RETURN then
			self.DeliveryState = ACraft.LAUNCH
			self.Waypoint = Vector(self.Pos.X, -500)	-- Go to orbit
		else -- Actor.AIMODE_STAY and Actor.AIMODE_DELIVER
			local FuturePos = self.Pos + self.Vel*20
			
			-- Make sure FuturePos is inside the scene
			if FuturePos.X > SceneMan.SceneWidth then
				if SceneMan.SceneWrapsX then
					FuturePos.X = FuturePos.X - SceneMan.SceneWidth
				else
					FuturePos.X = SceneMan.SceneWidth - self.Radius
				end
			elseif FuturePos.X < 0 then
				if SceneMan.SceneWrapsX then
					FuturePos.X = FuturePos.X + SceneMan.SceneWidth
				else
					FuturePos.X = self.Radius
				end
			end
			
			-- Use self:GetLastAIWaypoint() as a LZ so the AI can give orders to dropships
			local Wpt = self:GetLastAIWaypoint()
			if (self.Pos - Wpt).Largest > 1 then
				self.Waypoint = Wpt
			else
				local WptL = SceneMan:MovePointToGround(self.Pos-Vector(self.Radius, 0), self.hoverAlt, 12)
				local WptC = SceneMan:MovePointToGround(self.Pos, self.hoverAlt, 12)
				local WptR = SceneMan:MovePointToGround(self.Pos+Vector(self.Radius, 0), self.hoverAlt, 12)
				self.Waypoint = Vector(self.Pos.X, math.min(WptL.Y, WptC.Y, WptR.Y))
			end
			
			self.DeliveryState = ACraft.FALL
		end
	end
	
	if self.PlayerInterferedTimer:IsPastSimTimeLimit() then
		self.StuckTimer:Reset()
		
		local FuturePos = self.Pos + self.Vel*20
		
		-- Make sure FuturePos is inside the scene
		if FuturePos.X > SceneMan.SceneWidth then
			if SceneMan.SceneWrapsX then
				FuturePos.X = FuturePos.X - SceneMan.SceneWidth
			else
				FuturePos.X = SceneMan.SceneWidth - self.Radius
			end
		elseif FuturePos.X < 0 then
			if SceneMan.SceneWrapsX then
				FuturePos.X = FuturePos.X + SceneMan.SceneWidth
			else
				FuturePos.X = self.Radius
			end
		end
		
		local Dist = SceneMan:ShortestDistance(FuturePos, self.Waypoint, false)
		if math.abs(Dist.X) > 100 then
			if self.DeliveryState == ACraft.LAUNCH then
				self.Waypoint.X = FuturePos.X
				self.Waypoint.Y = -500
			else
				local WptL = SceneMan:MovePointToGround(self.Pos-Vector(self.Radius, 0), self.hoverAlt, 12)
				local WptC = SceneMan:MovePointToGround(self.Pos, self.hoverAlt, 12)
				local WptR = SceneMan:MovePointToGround(self.Pos+Vector(self.Radius, 0), self.hoverAlt, 12)
				self.Waypoint = Vector(self.Pos.X, math.min(WptL.Y, WptC.Y, WptR.Y))
			end
		end
	end
	
	self.PlayerInterferedTimer:Reset()
	
	-- Control right/left movement
	local Dist = SceneMan:ShortestDistance(self.Pos+self.Vel*30, self.Waypoint, false)
	local change = self.XposPID:Update(Dist.X, 0)
	if change > 2 then
		self.Ctrl.AnalogMove = Vector(change/30, 0)
	elseif change < -2 then
		self.Ctrl.AnalogMove = Vector(change/30, 0)
	end
	
	-- Control up/down movement
	Dist = SceneMan:ShortestDistance(self.Pos+self.Vel*5, self.Waypoint, false)
	change = self.YposPID:Update(Dist.Y, 0)
	if change > 2 then
		self.AltitudeMoveState = ACraft.DESCEND
	elseif change < -2 then
		self.AltitudeMoveState = ACraft.ASCEND
	end
	
	-- Delivery Sequence logic
	if self.DeliveryState == ACraft.FALL then
		-- Don't descend if we have nothing to deliver
		if self:IsInventoryEmpty() and self.AIMode ~= Actor.AIMODE_BRAINHUNT then
			if self.AIMode ~= Actor.AIMODE_STAY then
				self.DeliveryState = ACraft.LAUNCH
				self.HatchTimer:Reset()
				self.Waypoint.Y = -500	-- Go to orbit
			end
		else
			local dist = SceneMan:ShortestDistance(self.Pos, self.Waypoint, false).Magnitude
			if dist < self.Radius and math.abs(change) < 3 and math.abs(self.Vel.X) < 4 then	-- If we passed the hover check, check if we can start unloading
				local WptL = SceneMan:MovePointToGround(self.Pos+Vector(-self.Radius, -self.Radius), self.hoverAlt, 12)
				local WptC = SceneMan:MovePointToGround(self.Pos+Vector(0, -self.Radius), self.hoverAlt, 12)
				local WptR = SceneMan:MovePointToGround(self.Pos+Vector(self.Radius, -self.Radius), self.hoverAlt, 12)
				self.Waypoint = Vector(self.Pos.X, math.min(WptL.Y, WptC.Y, WptR.Y))
				
				dist = SceneMan:ShortestDistance(self.Pos, self.Waypoint, false).Magnitude
				if dist < self.Diameter then
					-- We are close enough to our waypoint
					if self.AIMode == Actor.AIMODE_STAY then
						self.DeliveryState = ACraft.STANDBY
					else
						self.DeliveryState = ACraft.UNLOAD
						self.HatchTimer:Reset()						
					end
				end
			else
				-- Check for something in the way of our descent, and hover to the side to avoid it
				if self.AvoidTimer:IsPastSimTimeLimit() then
					self.AvoidTimer:Reset()
					
					self.search = not self.search	-- Search every second update
					if self.search then
						local obstID = self:DetectObstacle(self.Diameter + self.Vel.Magnitude * 70)
						if obstID > 0 and obstID < 255 then
							local MO = MovableMan:GetMOFromID(MovableMan:GetRootMOID(obstID))
							if MO.ClassName == "ACDropShip" or MO.ClassName == "ACRocket" then
								self.AvoidMoveState = ACraft.HOVER
								self.Waypoint.X = self.Waypoint.X + self.Diameter * 2
								
								-- Make sure the LZ is inside the scene
								if self.Waypoint.X > SceneMan.SceneWidth then
									if SceneMan.SceneWrapsX then
										self.Waypoint.X = self.Waypoint.X - SceneMan.SceneWidth
									else
										self.Waypoint.X = SceneMan.SceneWidth - self.Radius
									end
								end
							end
						else
							self.AvoidMoveState = nil
						end
					else	-- Avoid terrain
						local Free = Vector()
						local Start = self.Pos + Vector(self.Radius, 0)
						local Trace = self.Vel * (self.Radius/2) + Vector(0,50)
						if PosRand() < 0.5 then
							Start.X = Start.X - self.Diameter
						end
						
						if SceneMan:CastStrengthRay(Start, Trace, 0, Free, 4, 0, true) then
							self.Waypoint.X = self.Pos.X
							self.Waypoint.Y = Free.Y - self.hoverAlt
						end
					end
				end
				
				if self.AvoidMoveState then
					self.AltitudeMoveState = self.AvoidMoveState
				end
			end
		end
	elseif self.DeliveryState == ACraft.UNLOAD then
		if self.HatchTimer:IsPastSimMS(500) then	-- Start unloading if there's something to unload
			self.HatchTimer:Reset()
			self:OpenHatch()
			
			if self.AIMode == Actor.AIMODE_BRAINHUNT and self:HasObjectInGroup("Brains") then
				self.AIMode = Actor.AIMODE_RETURN
			else
				self.DeliveryState = ACraft.FALL
			end
		end
	elseif self.DeliveryState == ACraft.LAUNCH then
		if self.HatchTimer:IsPastSimMS(1000) then
			self.HatchTimer:Reset()
			self:CloseHatch()
		end

		-- Check for something in the way of our ascent, and hover to the side to avoid it
		if self.AvoidTimer:IsPastSimTimeLimit() then
			self.AvoidTimer:Reset()
			
			local obstID = self:DetectObstacle(self.Diameter + self.Vel.Magnitude * 70)
			if obstID > 0 and obstID < 255 then
				local MO = MovableMan:GetMOFromID(MovableMan:GetRootMOID(obstID))
				if MO.ClassName == "ACDropShip" or MO.ClassName == "ACRocket" then
					self.AvoidMoveState = ACraft.HOVER
					self.Waypoint.X = self.Waypoint.X - self.Diameter * 2
					
					-- Make sure the LZ is inside the scene
					if self.Waypoint.X < 0 then
						if SceneMan.SceneWrapsX then
							self.Waypoint.X = self.Waypoint.X + SceneMan.SceneWidth
						else
							self.Waypoint.X = self.Radius
						end
					end
				end
			else
				self.AvoidMoveState = nil
			end
		end
		
		if self.AvoidMoveState then
			self.AltitudeMoveState = self.AvoidMoveState
		end
	end
	
	-- Input translation
	if self.AltitudeMoveState == ACraft.ASCEND then
		self.Ctrl:SetState(Controller.MOVE_UP, true)
	elseif self.AltitudeMoveState == ACraft.DESCEND then
		self.Ctrl:SetState(Controller.MOVE_DOWN, true)
	else
		self.Ctrl:SetState(Controller.MOVE_UP, false)
		self.Ctrl:SetState(Controller.MOVE_DOWN, false)
	end
	
	-- If we are hopelessly stuck, self destruct
	if self.Vel.Largest > 3 or self.AIMode == Actor.AIMODE_STAY then
		self.StuckTimer:Reset()
	elseif self.AIMode == Actor.AIMODE_SCUTTLE or self.StuckTimer:IsPastSimMS(40000) then
		self:GibThis()
	end
end
